from sqlalchemy import inspection
from sqlalchemy import orm
from sqlalchemy_mixins import serialize

from kit_up.core import models
from kit_up.core.contexts import uow
from kit_up.core.stashes import base, context_vars
from kit_up.core.domains import crud
from kit_up.core.data_mappers import base as mappers
from kit_up.impl.sql_alchemy import contexts
from kit_up.impl.sql_alchemy import repositories


class MyStash(base.AbstractAttrStash, context_vars.ContextVarStash):
    pass


class AppContext(contexts.SqlalchemyContextMixin, uow.UowContextMixin):
    pass


class AppDomain(crud.CrudDomain):
    pass


class AppRepo(repositories.SqlalchemyCrudRepository):
    pass


class AppModel(models.BaseModel):
    name: str
    surname: str


class SqlalchemyAdapterMixin(serialize.SerializeMixin):
    @classmethod
    def get_identity_qualifier(cls):
        result = []
        for pk in inspection.inspect(cls).primary_key:
            result.append(pk.name)

        if len(result) > 1:
            return result[0]
        elif result:
            return tuple(result)
        else:
            return None

    def get_identity(self):
        return getattr(self, self.get_identity_qualifier())

    def as_dict(self):
        return self.to_dict()


class BaseOrm(orm.DeclarativeBase, SqlalchemyAdapterMixin):
    pass


class AppOrm(BaseOrm):
    __tablename__ = "app_orm"

    name: orm.Mapped[str] = orm.MappedColumn(primary_key=True)
    surname: orm.Mapped[str] = orm.MappedColumn()


def main():
    stash = MyStash()
    ctx = AppContext(stash=stash)
    # uow = uow_fd.FdUow(context=ctx)
    repo = AppRepo(
        context=ctx,
        data_mapper=mappers.BaseDataMapper(AppModel, AppOrm),
    )
    domain = AppDomain(repository=repo)

    print("START: opening")
    ctx.uow_open()
    BaseOrm.metadata.create_all(ctx.sa_engine)
    print(f"OPENED: {ctx.uow}")
    views = domain.create(dict(name="John", surname="Johnson"))
    view = views[0]
    print(view)
    print(view.as_dict())
    print(domain.filter())
    ctx.uow.commit()
    print("pick")
    print(domain.pick("John"))
    print("get_by")
    print(domain.get_by(name="John"))
    print("filter_by")
    print(domain.filter_by(name="John", surname="Snow"))
    ctx.uow_close()
    print("CLOSED")

    use_case = ...
    access = ...


if __name__ == "__main__":
    main()
